@@ -62,6 +62,7 @@
       printSelect( array('Y' => 'Yes', 'N' => 'No'), 'hash', $val, "name='$key'");
     } elseif ($key == 'actions') {
       print "<tr><td></td><td>$pretty_key:</td><td>";
+      # TODO: port to printMultiSelect (and look for others to port too)
       print "\n<select multiple name='$key" .  "[]'>";
       $my_values = explode(",", $val);
       $my_options = array('autoreg' => 'Autoreg', 'email' => 'Email', 'log' => 'Log', 'trap' => 'Trap');
@@ -73,6 +74,10 @@
         }
       }
       print "</select>\n";
+    } elseif ($key == 'whitelisted_categories') {
+        print "<tr><td></td><td>$pretty_key:</td><td>";
+        $selected = explode(",", $val);
+        printMultiSelect(get_nodecategories_for_dropdown(), 'hash', $selected, "multiple name='{$key}[]'");
     } elseif ($key == 'trigger') {
       print "<tr><td></td><td>$pretty_key:</td><td><textarea name='$key'>$val</textarea>";
     } else {
