@@ -77,7 +77,7 @@ function validate_user_ldap($user,$pass,$hash='') {
     $filter="$ldap_user_key=$user";
   }
 
-  # We may need these 2 options
+  # We may have to set these 2 options
   #ldap_set_option($ldap, LDAP_OPT_REFERRALS, 0);
   #ldap_set_option($ldap, LDAP_OPT_PROTOCOL_VERSION, 3);
 
@@ -208,7 +208,7 @@ else {
 <table id="main" style="width: 100%;" cellpadding="0" cellspacing="0">
   <tbody><tr colspan="2">
     <td valign="top">
-      <a href="https://localhost:1443/index.php"><img src="/common/packetfence.png" alt="[ Packetfence ]" align="right" border="0" height="60" width="193"></a>
+      <a href="index.php"><img src="/common/packetfence.png" alt="[ Packetfence ]" align="right" border="0" height="60" width="193"></a>
     </td>
   </tr>
   <tr colspan="2">
